@@ -1,8 +1,12 @@
 from abc import ABCMeta
-from typing import Dict, Any, Union
+from typing import Union, TypeVar, Type
+
+from pydantic import BaseModel, ValidationError
+from requests import Session, RequestException
 
 from nidhoggr.core.response import ErrorResponse
-from requests import Session, RequestException
+
+T = TypeVar('T', bound=BaseModel)
 
 
 class RequestsRepo(metaclass=ABCMeta):
@@ -20,9 +24,9 @@ class RequestsRepo(metaclass=ABCMeta):
         self._api_url = api_url
         self._timeout = timeout
 
-    def fetch(self, *, endpoint: str, payload: Dict[str, Any]) -> Union[ErrorResponse, Dict[str, Any]]:
+    def fetch(self, *, endpoint: str, payload: BaseModel, clazz: Type[T]) -> Union[ErrorResponse, T]:
         try:
-            response = self._session.post(f'{self._api_url}{endpoint}', json=payload, timeout=self._timeout)
+            response = self._session.post(f'{self._api_url}{endpoint}', json=payload.dict(), timeout=self._timeout)
         except RequestException as e:
             return ErrorResponse(reason=f"Failed to fetch {endpoint}", exception=e)
 
@@ -31,5 +35,9 @@ class RequestsRepo(metaclass=ABCMeta):
 
         result = response.json()
         if not result:
-            return ErrorResponse(reason=f"Response from {endpoint} didn't contain valid JSON")
-        return result
+            return ErrorResponse(reason=f"Response from {endpoint} didn't contain valid JSON, but {response.text}")
+
+        try:
+            return clazz.parse_obj(result)
+        except ValidationError as e:
+            return ErrorResponse(reason=f"Received malformed {clazz} object from API, got {result}", exception=e)
